/*
 * OptParser.java
 *
 * Created on September 29, 2009, 5:21 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package splscli;

import java.lang.String;
import java.util.List;
import java.util.ArrayList;
import com.seapine.licenseserver.api.LSAPI.LicenseType;
import com.seapine.licenseserver.api.LicenseAssignment;

public class OptParser {
    
/**
 * Split the value from the given option string
 * @return The value portion of the name=option pair
 * @param option name=value pair to split
 * @author oliverm
 */
    public static String splitOpt(String option) {
        String[] optVal = option.split("=", 2);
        if(optVal.length > 1) {
            return optVal[1];
        } else {
            return "";
        }
    }
    
    /**
     * Parse licensing information from command line values
     * @return List of license type enumerations
     * @param value of "ttpro" option from command line
     * @param value of "tcm" option from command line
     * @param value of "ttrm" option from command line
     * @param value of "scm" option from command line
     */
    public static List<LicenseAssignment> parseLicenses(String ttpro, String ttpropool, String tcm,
            String tcmpool, String ttrm, String ttrmpool, String scm, String scmpool) {
        return parseLicenses(ttpro, ttpropool, tcm, tcmpool, ttrm, ttrmpool, scm, scmpool, null);
    }
    
    /**
     * Parse licensing information from command line values
     * @return List of license type enumerations
     * @param value of "ttpro" option from command line
     * @param value of "tcm" option from command line
     * @param value of "ttrm" option from command line
     * @param value of "scm" option from command line
     * @param list of licenses currently configured
     */
    public static List<LicenseAssignment> parseLicenses(String ttpro, String ttpropool, String tcm,
            String tcmpool, String ttrm, String ttrmpool, String scm, String scmpool,
            List<LicenseAssignment> inLicenses) {
        List<LicenseAssignment> licenses = inLicenses;
        
        if(licenses == null) {
            licenses = new ArrayList<LicenseAssignment>(4);
        }
        
        if(ttpro.equals("float")) {
            licenses.add(new LicenseAssignment(LicenseType.ttpFloating, Long.parseLong(ttpropool)));
        } else if(ttpro.equals("named")) {
            licenses.add(new LicenseAssignment(LicenseType.ttpNamed, Long.parseLong(ttpropool)));
        } else if(ttpro.equals("none")) {
            LicenseAssignment floatRemove = null;
            LicenseAssignment namedRemove = null;
            for(final LicenseAssignment licAsgn : licenses) {
                if(licAsgn.licenseType == LicenseType.ttpFloating) {
                    floatRemove = licAsgn;
                } else if(licAsgn.licenseType == LicenseType.ttpNamed) {
                    namedRemove = licAsgn;
                }
            }

            if(floatRemove != null) {
                licenses.remove(floatRemove);
            }

            if(namedRemove != null) {
                licenses.remove(namedRemove);
            }
        }
        
        if(tcm.equals("float")) {
            licenses.add(new LicenseAssignment(LicenseType.tcmFloating, Long.parseLong(ttpropool)));
        } else if(tcm.equals("named")) {
            licenses.add(new LicenseAssignment(LicenseType.tcmNamed, Long.parseLong(ttpropool)));
        } else if(tcm.equals("none")) {
            LicenseAssignment floatRemove = null;
            LicenseAssignment namedRemove = null;
            for(final LicenseAssignment licAsgn : licenses) {
                if(licAsgn.licenseType == LicenseType.tcmFloating) {
                    floatRemove = licAsgn;
                } else if(licAsgn.licenseType == LicenseType.tcmNamed) {
                    namedRemove = licAsgn;
                }
            }

            if(floatRemove != null) {
                licenses.remove(floatRemove);
            }

            if(namedRemove != null) {
                licenses.remove(namedRemove);
            }
        }
        
        if(ttrm.equals("float")) {
            licenses.add(new LicenseAssignment(LicenseType.rmFloating, Long.parseLong(ttpropool)));
        } else if(ttrm.equals("named")) {
            licenses.add(new LicenseAssignment(LicenseType.rmNamed, Long.parseLong(ttpropool)));
        } else if(ttrm.equals("none")) {
            LicenseAssignment floatRemove = null;
            LicenseAssignment namedRemove = null;
            for(final LicenseAssignment licAsgn : licenses) {
                if(licAsgn.licenseType == LicenseType.rmFloating) {
                    floatRemove = licAsgn;
                } else if(licAsgn.licenseType == LicenseType.rmNamed) {
                    namedRemove = licAsgn;
                }
            }

            if(floatRemove != null) {
                licenses.remove(floatRemove);
            }

            if(namedRemove != null) {
                licenses.remove(namedRemove);
            }
        }
        
        if(scm.equals("float")) {
            licenses.add(new LicenseAssignment(LicenseType.scmFloating, Long.parseLong(ttpropool)));
        } else if(scm.equals("named")) {
            licenses.add(new LicenseAssignment(LicenseType.scmNamed, Long.parseLong(ttpropool)));
        } else if(scm.equals("none")) {
            LicenseAssignment floatRemove = null;
            LicenseAssignment namedRemove = null;
            for(final LicenseAssignment licAsgn : licenses) {
                if(licAsgn.licenseType == LicenseType.scmFloating) {
                    floatRemove = licAsgn;
                } else if(licAsgn.licenseType == LicenseType.scmNamed) {
                    namedRemove = licAsgn;
                }
            }

            if(floatRemove != null) {
                licenses.remove(floatRemove);
            }

            if(namedRemove != null) {
                licenses.remove(namedRemove);
            }
        }
        
        return licenses;
    }

    /**
     * Parse licensing information from command line values
     * @return List of license type enumerations
     * @param value of "ttpro" option from command line
     * @param value of "tcm" option from command line
     * @param value of "ttrm" option from command line
     * @param value of "scm" option from command line
     * @param list of licenses currently configured
     */
    public static List<LicenseType> parseLicenses(String ttpro, String tcm, String ttrm,
            String scm) {
        List<LicenseType> licenses = new ArrayList<LicenseType>(4);

        if(ttpro.equals("float")) {
            licenses.add(LicenseType.ttpFloating);
        } else if(ttpro.equals("named")) {
            licenses.add(LicenseType.ttpNamed);
        } else if(ttpro.equals("none")) {
            licenses.remove(LicenseType.ttpFloating);
            licenses.remove(LicenseType.ttpNamed);
        }

        if(tcm.equals("float")) {
            licenses.add(LicenseType.tcmFloating);
        } else if(tcm.equals("named")) {
            licenses.add(LicenseType.tcmNamed);
        } else if(tcm.equals("none")) {
            licenses.remove(LicenseType.tcmFloating);
            licenses.remove(LicenseType.tcmNamed);
        }

        if(ttrm.equals("float")) {
            licenses.add(LicenseType.rmFloating);
        } else if(ttrm.equals("named")) {
            licenses.add(LicenseType.rmNamed);
        } else if(ttrm.equals("none")) {
            licenses.remove(LicenseType.rmFloating);
            licenses.remove(LicenseType.rmNamed);
        }

        if(scm.equals("float")) {
            licenses.add(LicenseType.scmFloating);
        } else if(scm.equals("named")) {
            licenses.add(LicenseType.scmNamed);
        } else if(scm.equals("none")) {
            licenses.remove(LicenseType.scmFloating);
            licenses.remove(LicenseType.scmNamed);
        }

        return licenses;
    }
}
