using System;
using System.Collections.Generic;
using System.Text;
using LSAdminAPI;

namespace splscli
{
    class splscli
    {
        private string mUsername;
        private string mName;
        private string mLDAPServer;
        private string mTTPro;
        private string mTCM;
        private string mTTRM;
        private string mSCM;
        private SecurityLevels mSecurityLevel;
        private string mLDAPUsername;
        private List<string> mUsers;
        private string mSerialNumber;
        private UInt32 mID;
        private UInt32 mFromID;
        private UInt32 mToID;
        private UInt32 mSNID;
        private int mCount;

        private string mCredentials;
        private string mCommand;
        private LSAdminAPI.LSAdminAPI mAPI;
        private List<string> mArgs;

        splscli(string[] args)
        {
            mAPI = new LSAdminAPI.LSAdminAPI();
            mArgs = parseArgs(args);
        }

        public static void printHelp()
        {
            Console.WriteLine("Usage: splscli [[username[:password]]@hostname[:port]] [command] [options]");
            Console.WriteLine("Commands:");
            Console.WriteLine("  getlsversion  Get version of license server");
            Console.WriteLine("  getavailablefloating  Get list of available floating license counts");
            Console.WriteLine("  getusedfloating  Get list of used floating license counts");
            Console.WriteLine("  importuser  Import a user from LDAP");
            Console.WriteLine("     --username    pattern to match user's username");
            Console.WriteLine("     --name        pattern to match user's name");
            Console.WriteLine("     --ldapserver  ldap server from which to import user");
            Console.WriteLine("     --security    License Server security level (none, fetch, users, full)");
            Console.WriteLine("     --ttpro       TestTrack Pro license type (none, float, named)");
            Console.WriteLine("     --tcm         TestTrack TCM license type (none, float, named)");
            Console.WriteLine("     --ttrm        TestTrack RM license type (none, float, named)");
            Console.WriteLine("     --scm         Surround SCM license type (none, float, named)");
            Console.WriteLine("  associateuser  Associate a License Server user to an LDAP user\n");
            Console.WriteLine("     --lsusername             user's username in License Server\n");
            Console.WriteLine("     --ldapusername           user's username in LDAP (if not specified, lsusername will be used)\n");
            Console.WriteLine("     --ldapserver             LDAP server in which to associate user\n");
            Console.WriteLine("  adduser     Add a user to License Server");
            Console.WriteLine("     --firstname   user's first name");
            Console.WriteLine("     --initials    user's initials");
            Console.WriteLine("     --lastname    user's last name");
            Console.WriteLine("     --username    user's username");
            Console.WriteLine("     --password    user's password");
            Console.WriteLine("     --active                 whether user is active or inactive (yes, no)\n");
            Console.WriteLine("     --emailtype              type of user's email address (smtp, mapi, other)\n");
            Console.WriteLine("     --email       user's email address");
            Console.WriteLine("     --phone1type             type of user's phone number (work, fax, home, pager, mobile)\n");
            Console.WriteLine("     --phone1                 user's phone number\n");
            Console.WriteLine("     --phone2type             type of user's phone number (work, fax, home, pager, mobile)\n");
            Console.WriteLine("     --phone2                 user's phone number\n");
            Console.WriteLine("     --forcechangepassword    whether the user must change his password at next login (yes, no)\n");
            Console.WriteLine("     --preventchangepassword  whether the user is prevented from changing his password (yes, no)\n");
            Console.WriteLine("     --type                   type of user (user, customer)\n");
            Console.WriteLine("     --address     user's address");
            Console.WriteLine("     --company     user's company");
            Console.WriteLine("     --division    user's division");
            Console.WriteLine("     --department  user's department");
            Console.WriteLine("     --security    License Server security level (none, fetch, users, full)");
            Console.WriteLine("     --singlesignon           whether the user is permitted to login using single sign-on (yes, no)\n");
            Console.WriteLine("     --ttpro       TestTrack Pro license type (none, float, named)");
            Console.WriteLine("     --ttpropool   TestTrack Pro floating license pool");
            Console.WriteLine("     --tcm         TestTrack TCM license type (none, float, named)");
            Console.WriteLine("     --tcmpool                TestTrack TCM floating license pool");
            Console.WriteLine("     --ttrm        TestTrack RM license type (none, float, named)");
            Console.WriteLine("     --ttrmpool               TestTrack RM floating license pool");
            Console.WriteLine("     --scm         Surround SCM license type (none, float, named)");
            Console.WriteLine("     --scmpool                Surround SCM floating license pool");
            Console.WriteLine("     --notes                  user notes\n");
            Console.WriteLine("  listusers      List all users in License Server\n");
            Console.WriteLine("     --users                  comma-delimited list of usernames to list\n");
            Console.WriteLine("  edituser       Modify a user in License Server\n");
            Console.WriteLine("     --username               username of user to modify\n");
            Console.WriteLine("     --firstname   user's first name");
            Console.WriteLine("     --initials    user's initials");
            Console.WriteLine("     --lastname    user's last name");
            Console.WriteLine("     --newusername            new username for user\n");
            Console.WriteLine("     --password    user's password");
            Console.WriteLine("     --active                 whether user is active or inactive (yes, no)\n");
            Console.WriteLine("     --emailtype              type of user's email address (smtp, mapi, other)\n");
            Console.WriteLine("     --email       user's email address");
            Console.WriteLine("     --phone1type             type of user's phone number (work, fax, home, pager, mobile)\n");
            Console.WriteLine("     --phone1                 user's phone number\n");
            Console.WriteLine("     --phone2type             type of user's phone number (work, fax, home, pager, mobile)\n");
            Console.WriteLine("     --phone2                 user's phone number\n");
            Console.WriteLine("     --forcechangepassword    whether the user must change his password at next login (yes, no)\n");
            Console.WriteLine("     --preventchangepassword  whether the user is prevented from changing his password (yes, no)\n");
            Console.WriteLine("     --type                   type of user (user, customer)\n");
            Console.WriteLine("     --address     user's address");
            Console.WriteLine("     --company     user's company");
            Console.WriteLine("     --division    user's division");
            Console.WriteLine("     --department  user's department");
            Console.WriteLine("     --security    License Server security level (none, fetch, users, full)");
            Console.WriteLine("     --singlesignon           whether the user is permitted to login using single sign-on (yes, no)\n");
            Console.WriteLine("     --ttpro       TestTrack Pro license type (none, float, named)");
            Console.WriteLine("     --ttpropool   TestTrack Pro floating license pool");
            Console.WriteLine("     --tcm         TestTrack TCM license type (none, float, named)");
            Console.WriteLine("     --tcmpool                TestTrack TCM floating license pool");
            Console.WriteLine("     --ttrm        TestTrack RM license type (none, float, named)");
            Console.WriteLine("     --ttrmpool               TestTrack RM floating license pool");
            Console.WriteLine("     --scm         Surround SCM license type (none, float, named)");
            Console.WriteLine("     --scmpool                Surround SCM floating license pool");
            Console.WriteLine("     --notes                  user notes");
            Console.WriteLine("  addsn          Add a serial number to License Server.");
            Console.WriteLine("     --serialnumber           serial number to add to License Server");
            Console.WriteLine("  listsns        Display a list of serial number information.  Each line is tab-delimited.");
            Console.WriteLine("  deletesn       Remove a serial number from License Server.");
            Console.WriteLine("     --id                     unique identifier of serial number to remove from License Server");
            Console.WriteLine("  addpool        Add a pool to License Server.");
            Console.WriteLine("    --name                    pool's name");
            Console.WriteLine("    --product                 product for which pool will contain licenses (ttpro, tcm, ttrm, scm)");
            Console.WriteLine("  listpools      Display list of pool information.  Each line is tab-delimited.");
            Console.WriteLine("  editpool       Modify pool in License Server.");
            Console.WriteLine("    --id                      unique identifer of pool to modify");
            Console.WriteLine("    --name                    new name for pool");
            Console.WriteLine("    --product                 new product for pool (ttpro, tcm, ttrm, scm)  Note: changing product will move all licenses to the default pool.");
            Console.WriteLine("  movelicenses   Move licenses between pools in License Server");
            Console.WriteLine("    --fromid                  unique identifier of pool from which to remove licenses");
            Console.WriteLine("    --toid                    unique identifier of pool to which to add licenses");
            Console.WriteLine("    --snid                    unique identifier of serial number for which to move licenses");
            Console.WriteLine("    --count                   number of licenses to move");
            Console.WriteLine("  deletepool     Remove a pool from License Server.");
            Console.WriteLine("    --id                      unique identifier of pool to delete");
            Console.WriteLine("  -h  --help");
            Console.WriteLine("  -V  --version");
        }

        public void executeCommand()
        {
            string lsUser = null;
            string lsPassword = null;
            string lsHost = null;
            int lsPort = 0;
            List<LicenseTypes> licenses = new List<LicenseTypes>();

            if (mCommand == "getlsversion")
            {
               parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

               mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
               int version = mAPI.GetLicenseServerVersion();
               Console.WriteLine("License Server version: " + version);
               mAPI.Disconnect();
            } 
            else if (mCommand == "importuser")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);
                licenses = parseLicenses(mTTPro, mTCM, mTTRM, mSCM);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
                mAPI.ImportLDAPUser(mName, mUsername, mLDAPServer, mSecurityLevel, licenses);
                mAPI.Disconnect();
            }
            else if (mCommand == "getavailablefloating")
            {
               parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);
               licenses = parseLicenses(mTTPro, mTCM, mTTRM, mSCM);

               mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
               List<CLicenseCount> myLicenseUsageList = mAPI.GetAvailableFloatingLicenses();
               Console.WriteLine("Available licenses: ");
               foreach (CLicenseCount license in myLicenseUsageList)
               {
                  Console.WriteLine(license.Product + " " + license.SerialNumberType + " " + license.Count);
               }
               mAPI.Disconnect();
            }
            else if (mCommand == "getusedfloating")
            {
               parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);
               licenses = parseLicenses(mTTPro, mTCM, mTTRM, mSCM);

               mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
               List<CLicenseCount> myLicenseUsageList = mAPI.GetUsedFloatingLicenses();
               Console.WriteLine("Used licenses: ");
               foreach (CLicenseCount license in myLicenseUsageList)
               {
                  Console.WriteLine(license.Product + " " + license.SerialNumberType + " " + license.Count);
               }
               mAPI.Disconnect();
            }
            else if (mCommand == "associateuser")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
                mAPI.AssociateToLDAPUser(mUsername, mLDAPUsername, mLDAPServer);
                mAPI.Disconnect();
            }
            else if (mCommand == "adduser")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);
                licenses = parseLicenses(mTTPro, mTCM, mTTRM, mSCM);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
                CGlobalUser user = parseUser(mArgs, null);
                mAPI.AddUserEx(user);
                mAPI.Disconnect();
            }
            else if (mCommand == "listusers")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
                List<CGlobalUser> users = mAPI.GetUsers(mUsers);

                Console.WriteLine(CGlobalUser.DisplayHeader());

                foreach (CGlobalUser user in users)
                {
                    Console.WriteLine(user.ToString());
                }

                mAPI.Disconnect();
            }
            else if (mCommand == "edituser")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);

                List<string> usernames = new List<string>(1);
                usernames.Add(mUsername);

                List<CGlobalUser> users = mAPI.GetUsers(usernames);

                if (users.Count < 1)
                {
                    throw new Exception("Could not find user: " + mUsername);
                }
                else if (users.Count > 1)
                {
                    throw new Exception("Did not specify a unique user");
                }

                CGlobalUser updatedUser = parseUser(mArgs, users[0]);
                mAPI.EditUser(mUsername, updatedUser);

                mAPI.Disconnect();
            }
            else if (mCommand == "addsn")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
                mAPI.AddSerialNumber(mSerialNumber);
                mAPI.Disconnect();
            }
            else if (mCommand == "listsns")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);

                List<CSerialNumber> sns = mAPI.GetSerialNumbers();

                Console.WriteLine(CSerialNumber.DisplayHeader());

                foreach (CSerialNumber sn in sns)
                {
                    Console.WriteLine(sn.ToString());
                }

                mAPI.Disconnect();
            }
            else if (mCommand == "deletesn")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
                mAPI.DeleteSerialNumber(mID);
                mAPI.Disconnect();
            }
            else if (mCommand == "addpool")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);
                CPool pool = parsePool(mArgs, null);
                mAPI.AddPool(pool);
                mAPI.Disconnect();
            }
            else if (mCommand == "listpools")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);

                List<CPool> pools = mAPI.GetPools();

                Console.WriteLine(CPool.DisplayHeader());

                foreach (CPool pool in pools)
                {
                    Console.WriteLine(pool.ToString());
                }

                mAPI.Disconnect();
            }
            else if (mCommand == "editpool")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);

                List<CPool> pools = mAPI.GetPools();

                CPool editPool = null;

                foreach (CPool pool in pools)
                {
                    if (pool.ID == mID)
                    {
                        editPool = pool;
                        break;
                    }
                }

                if (editPool == null)
                {
                    throw new ArgumentException("Specified pool does not exist in License Server");
                }

                List<CPool> savePools = new List<CPool>(1);
                savePools.Add(parsePool(mArgs, editPool));

                mAPI.EditPools(savePools);

                mAPI.Disconnect();
            }
            else if (mCommand == "movelicenses")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);

                List<CPool> pools = mAPI.GetPools();
                CPool[] editPools = new CPool[2];
                const uint FROM = 0;
                const uint TO = 1;

                // find the pools to modify
                foreach (CPool pool in pools)
                {
                    if (pool.ID == mFromID)
                    {
                        editPools[FROM] = pool;
                    }
                    else if (pool.ID == mToID)
                    {
                        editPools[TO] = pool;
                    }

                    if (editPools[FROM] != null && editPools[TO] != null)
                        break;
                }

                if (editPools[FROM] == null || editPools[TO] == null)
                {
                    throw new ArgumentException("Specified pool does not exist in License Server");
                }

                // remove the licenses from the source pool
                bool found = false;
                CSerialNumCounter removeCounter = null;
                foreach (CSerialNumCounter snCount in editPools[FROM].SerialNumbers)
                {
                    if (snCount.SNID == mSNID)
                    {
                        if (snCount.Count > mCount)
                        {
                            found = true;
                            snCount.Count = snCount.Count - mCount;
                        }
                        else if (snCount.Count < mCount)
                        {
                            throw new ArgumentException("Invalid serial number count specified");
                        }
                        else
                        {
                            removeCounter = snCount;
                        }

                        break;
                    }
                }

                if (!found)
                {
                    throw new ArgumentException("Specified serial number is not in source pool");
                }

                if (removeCounter != null)
                {
                    editPools[FROM].SerialNumbers.Remove(removeCounter);
                }

                // add the licenses to the destination pool
                found = false;
                foreach (CSerialNumCounter snCount in editPools[TO].SerialNumbers)
                {
                    if (snCount.SNID == mSNID)
                    {
                        snCount.Count = snCount.Count + mCount;
                        found = true;
                        break;
                    }
                }

                if (!found)
                {
                    editPools[TO].SerialNumbers.Add(new CSerialNumCounter(mSNID, mCount));
                }

                mAPI.EditPools(new List<CPool>(editPools));

                mAPI.Disconnect();
            }
            else if (mCommand == "deletepool")
            {
                parseCredentials(ref lsUser, ref lsPassword, ref lsHost, ref lsPort);

                mAPI.Connect(lsHost, lsPort, lsUser, lsPassword);

                mAPI.DeletePool(mID);

                mAPI.Disconnect();
            }
            else if (mCommand == "HELP")
            {
                printHelp();
            }
            else if (mCommand == "VERSION")
            {
                Console.WriteLine("splscli: Helix ALM License Server Command Line Tool 1.0.0");
            }
            else
            {
                throw new ArgumentException("Unrecognized command: " + mCommand);
            }
        }

        private void parseCredentials(ref string username, ref string password, ref string host, ref int port)
        {
            string[] outerParts = mCredentials.Split('@');
            string userpass = "";
            string hostport = "";

            if (outerParts.Length > 1)
            {
                userpass = outerParts[0];
                hostport = outerParts[1];
            }
            else
            {
                hostport = outerParts[0];
            }

            if (userpass != "")
            {
                string[] loginParts = userpass.Split(':');

                username = loginParts[0];
                if (loginParts.Length > 1)
                {
                    password = loginParts[1];
                }
            }

            if (hostport != "")
            {
                string[] hostParts = hostport.Split(':');

                host = hostParts[0];
                if (hostParts.Length > 1)
                {
                    port = Convert.ToInt32(hostParts[1]);
                }
                else
                {
                    port = 5100;
                }
                
            }
        }

        private List<string> parseArgs(string[] args)
        {
            List<string> remainingArgs = new List<string>(args.Length);

            foreach (string arg in args)
            {
                if (arg[0] != '-')
                {
                    // either credentials or command, and credentials come first
                    if (mCredentials == null)
                    {
                        mCredentials = arg;
                    }
                    else if (mCommand == null)
                    {
                        mCommand = arg;
                    }
                    else
                    {
                        // malformed parameters...
                        throw new ArgumentException("Unexpected parameter: " + arg);
                    }
                }
                else
                {
                    // command modifiers
                    if (arg == "-h" || arg == "--help")
                    {
                        mCommand = "HELP";
                    }
                    else if (arg == "-V" || arg == "--version")
                    {
                        mCommand = "VERSION";
                    }
                    // command data
                    else if (arg.Contains("--username"))
                    {
                        mUsername = splitOption(arg);
                        // keep for user parsing
                        remainingArgs.Add(arg);
                    }
                    else if (arg.Contains("--name"))
                    {
                        mName = splitOption(arg);
                        // keep for pool parsing
                        remainingArgs.Add(arg);
                    }
                    else if (arg.Contains("--ldapserver"))
                    {
                        mLDAPServer = splitOption(arg);
                    }
                    else if (arg.Contains("--lsusername"))
                    {
                        mUsername = splitOption(arg);
                    }
                    else if (arg.Contains("--ldapusername"))
                    {
                        mLDAPUsername = splitOption(arg);
                    }
                    else if (arg.Contains("--ttpro"))
                    {
                        mTTPro = splitOption(arg);
                        // keep for user parsing
                        remainingArgs.Add(arg);
                    }
                    else if (arg.Contains("--tcm"))
                    {
                        mTCM = splitOption(arg);
                        // keep for user parsing
                        remainingArgs.Add(arg);
                    }
                    else if (arg.Contains("--ttrm"))
                    {
                        mTTRM = splitOption(arg);
                        // keep for user parsing
                        remainingArgs.Add(arg);
                    }
                    else if (arg.Contains("--scm"))
                    {
                        mSCM = splitOption(arg);
                        // keep for user parsing
                        remainingArgs.Add(arg);
                    }
                    else if (arg.Contains("--security"))
                    {
                        mSecurityLevel = parseSecurityLevel(splitOption(arg));
                        // keep for user parsing
                        remainingArgs.Add(arg);
                    }
                    else if (arg.Contains("--users"))
                    {
                        mUsers = new List<string>(splitOption(arg).Split(','));
                    }
                    else if (arg.Contains("--serialnumber"))
                    {
                        mSerialNumber = splitOption(arg);
                    }
                    else if (arg.Contains("--id"))
                    {
                        mID = System.Convert.ToUInt32(splitOption(arg));
                        // keep for pool parsing
                        remainingArgs.Add(arg);
                    }
                    else if (arg.Contains("--fromid"))
                    {
                        mFromID = System.Convert.ToUInt32(splitOption(arg));
                    }
                    else if (arg.Contains("--toid"))
                    {
                        mToID = System.Convert.ToUInt32(splitOption(arg));
                    }
                    else if (arg.Contains("--snid"))
                    {
                        mSNID = System.Convert.ToUInt32(splitOption(arg));
                    }
                    else if (arg.Contains("--count"))
                    {
                        mCount = System.Convert.ToInt32(splitOption(arg));
                    }
                    else
                    {
                        // unknown arguments should be kept for user parsing
                        remainingArgs.Add(arg);
                    }
                }
            }

            return remainingArgs;
        }

        private static string splitOption(string option)
        {
            string value = "";
            string[] parts = option.Split('=');
            if (parts.Length > 1)
            {
                value = parts[1];
            }

            return value;
        }

        static bool parseYesNo(string val)
        {
            bool parsed = false;

            if (val.Contains("yes"))
            {
                parsed = true;
            }

            return parsed;
        }

        static private PhoneTypes parsePhoneType(string type)
        {
            PhoneTypes parsed = PhoneTypes.kWorkPhone;

            if (type.Contains("fax"))
            {
                parsed = PhoneTypes.kFaxPhone;
            }
            else if (type.Contains("home"))
            {
                parsed = PhoneTypes.kHomePhone;
            }
            else if (type.Contains("pager"))
            {
                parsed = PhoneTypes.kPagerPhone;
            }
            else if (type.Contains("mobile"))
            {
                parsed = PhoneTypes.kMobilePhone;
            }

            return parsed;
        }

        static private EmailTypes parseEmailType(string type)
        {
            EmailTypes parsed = EmailTypes.kInternetEmail;

            if (type.Contains("mapi"))
            {
                parsed = EmailTypes.kMAPIEmail;
            }
            else if (type.Contains("other"))
            {
                parsed = EmailTypes.kOtherEmail;
            }

            return parsed;
        }

        static private UserTypes parseUserType(string type)
        {
            UserTypes parsed = UserTypes.kUser;

            if (type.Contains("customer"))
            {
                parsed = UserTypes.kCustomer;
            }

            return parsed;
        }

        static private SecurityLevels parseSecurityLevel(string level)
        {
            SecurityLevels secLevel = SecurityLevels.kNoAccess;

            if (level.Contains("fetch"))
            {
                secLevel = SecurityLevels.kRetrieveUsers;
            }
            else if (level.Contains("users"))
            {
                secLevel = SecurityLevels.kManageUsers;
            }
            else if (level.Contains("full"))
            {
                secLevel = SecurityLevels.kFullControl;
            }

            return secLevel;
        }

        static private List<LicenseTypes> parseLicenses(string ttpro, string tcm, string ttrm, string scm)
        {
            List<CLicenseAssignment> assignments = parseLicenses(ttpro, null, tcm, null, ttrm, null, scm, null, null);
            List<LicenseTypes> types = new List<LicenseTypes>(assignments.Count);

            foreach (CLicenseAssignment assign in assignments)
            {
                types.Add(assign.LicenseType);
            }

            return types;
        }

        static private List<CLicenseAssignment> parseLicenses(string ttpro, string ttpropool, string tcm, string tcmpool, string ttrm,
            string ttrmpool, string scm, string scmpool, List<CLicenseAssignment> inLicenses)
        {
            List<CLicenseAssignment> licenses = null;

            if (inLicenses == null)
            {
                licenses = new List<CLicenseAssignment>(4);
            }
            else
            {
                licenses = new List<CLicenseAssignment>(inLicenses);
            }

            if (ttpro != null)
            {
                uint poolID = 4294967295;

                if (ttpropool != null && ttpropool.Length > 0)
                {
                    poolID = uint.Parse(ttpropool);
                }

                if (ttpro == "float")
                {
                    licenses.Add(new CLicenseAssignment(LicenseTypes.kTTPFloating, poolID));
                }
                else if (ttpro == "named")
                {
                    licenses.Add(new CLicenseAssignment(LicenseTypes.kTTPNamed, 0));
                }
                else if (ttpro == "none")
                {
                    List<int> toRemove = new List<int>();

                    // find and remove any ttpro license types
                    for(int index = 0; index < licenses.Count; index++)
                    {
                        if(licenses[index].LicenseType == LicenseTypes.kTTPFloating ||
                           licenses[index].LicenseType == LicenseTypes.kTTPNamed)
                        {
                            toRemove.Add(index);
                        }
                    }
                    
                    foreach(int i in toRemove) 
                    {
                        licenses.RemoveAt(i);
                    }
                }
            }

            if (tcm != null)
            {
                uint poolID = 4294967295;

                if (tcmpool != null && tcmpool.Length > 0)
                {
                    poolID = uint.Parse(tcmpool);
                }

                if (tcm == "float")
                {
                    licenses.Add(new CLicenseAssignment(LicenseTypes.kTCMFloating, poolID));
                }
                else if (tcm == "named")
                {
                    licenses.Add(new CLicenseAssignment(LicenseTypes.kTCMNamed, 0));
                }
                else if (tcm == "none")
                {
                    List<int> toRemove = new List<int>();

                    // find and remove any ttpro license types
                    for (int index = 0; index < licenses.Count; index++)
                    {
                        if (licenses[index].LicenseType == LicenseTypes.kTCMFloating ||
                           licenses[index].LicenseType == LicenseTypes.kTCMNamed)
                        {
                            toRemove.Add(index);
                        }
                    }

                    foreach (int i in toRemove)
                    {
                        licenses.RemoveAt(i);
                    }
                }
            }

            if (ttrm != null)
            {
                uint poolID = 4294967295;

                if (ttrmpool != null && ttrmpool.Length > 0)
                {
                    poolID = uint.Parse(ttrmpool);
                }

                if (ttrm == "float")
                {
                    licenses.Add(new CLicenseAssignment(LicenseTypes.kRMFloating, poolID));
                }
                else if (ttrm == "named")
                {
                    licenses.Add(new CLicenseAssignment(LicenseTypes.kRMNamed, 0));
                }
                else if (ttrm == "none")
                {
                    List<int> toRemove = new List<int>();

                    // find and remove any ttpro license types
                    for (int index = 0; index < licenses.Count; index++)
                    {
                        if (licenses[index].LicenseType == LicenseTypes.kRMFloating ||
                           licenses[index].LicenseType == LicenseTypes.kRMNamed)
                        {
                            toRemove.Add(index);
                        }
                    }

                    foreach (int i in toRemove)
                    {
                        licenses.RemoveAt(i);
                    }
                }
            }

            if (scm != null)
            {
                uint poolID = 4294967295;

                if (scmpool != null && scmpool.Length > 0)
                {
                    poolID = uint.Parse(scmpool);
                }

                if (scm == "float")
                {
                    licenses.Add(new CLicenseAssignment(LicenseTypes.kSCMFloating, poolID));
                }
                else if (scm == "named")
                {
                    licenses.Add(new CLicenseAssignment(LicenseTypes.kSCMNamed, 0));
                }
                else if (scm == "none")
                {
                    List<int> toRemove = new List<int>();

                    // find and remove any ttpro license types
                    for (int index = 0; index < licenses.Count; index++)
                    {
                        if (licenses[index].LicenseType == LicenseTypes.kSCMFloating ||
                           licenses[index].LicenseType == LicenseTypes.kSCMNamed)
                        {
                            toRemove.Add(index);
                        }
                    }

                    foreach (int i in toRemove)
                    {
                        licenses.RemoveAt(i);
                    }
                }
            }

            return licenses;
        }

        static private CGlobalUser parseUser(List<string> args, CGlobalUser inUser)
        {
            CGlobalUser user = null;

            if (inUser == null)
            {
                user = new CGlobalUser();
            }
            else
            {
                user = new CGlobalUser(inUser);
            }

            string ttpro = "";
            string tcm = "";
            string ttrm = "";
            string scm = "";
            string ttpropool = "";
            string tcmpool = "";
            string ttrmpool = "";
            string scmpool = "";

            bool usernameSet = false;

            foreach(string arg in args)
            {
                if (arg.Contains("--firstname"))
                {
                    user.FirstName = splitOption(arg);
                }
                else if (arg.Contains("--initials"))
                {
                    user.Initials = splitOption(arg);
                }
                else if (arg.Contains("--lastname"))
                {
                    user.LastName = splitOption(arg);
                }
                else if (arg.Contains("--username"))
                {
                    if (!usernameSet)
                    {
                        user.Username = splitOption(arg);
                    }
                }
                else if (arg.Contains("--newusername"))
                {
                    user.Username = splitOption(arg);
                    usernameSet = true;
                }
                else if (arg.Contains("--password"))
                {
                    user.Password = splitOption(arg);
                }
                else if (arg.Contains("--active"))
                {
                    user.IsActive = parseYesNo(splitOption(arg));
                }
                else if (arg.Contains("--emailtype"))
                {
                    user.EmailAddressType = parseEmailType(splitOption(arg));
                }
                else if (arg.Contains("--email"))
                {
                    user.EmailAddress = splitOption(arg);
                }
                else if (arg.Contains("--phone1type"))
                {
                    user.PhoneNumber1Type = parsePhoneType(splitOption(arg));
                }
                else if (arg.Contains("--phone1"))
                {
                    user.PhoneNumber1 = splitOption(arg);
                }
                else if (arg.Contains("--phone2type"))
                {
                    user.PhoneNumber2Type = parsePhoneType(splitOption(arg));
                }
                else if (arg.Contains("--phone2"))
                {
                    user.PhoneNumber2 = splitOption(arg);
                }
                else if (arg.Contains("--forcechangepassword"))
                {
                    user.ChangePasswordAtLogon = parseYesNo(splitOption(arg));
                }
                else if (arg.Contains("--preventchangepassword"))
                {
                    user.CannotChangePassword = parseYesNo(splitOption(arg));
                }
                else if (arg.Contains("--type"))
                {
                    user.UserType = parseUserType(splitOption(arg));
                }
                else if (arg.Contains("--address"))
                {
                    user.Address = splitOption(arg);
                }
                else if (arg.Contains("--company"))
                {
                    user.Company = splitOption(arg);
                }
                else if (arg.Contains("--division"))
                {
                    user.Division = splitOption(arg);
                }
                else if (arg.Contains("--department"))
                {
                    user.Department = splitOption(arg);
                }
                else if (arg.Contains("--security"))
                {
                    user.SecurityLevel = parseSecurityLevel(splitOption(arg));
                }
                else if (arg.Contains("--singlesignon"))
                {
                    user.SingleSignOn = parseYesNo(splitOption(arg));
                }
                else if (arg.Contains("--ttpro"))
                {
                    ttpro = splitOption(arg);
                }
                else if (arg.Contains("--ttpropool"))
                {
                    ttpropool = splitOption(arg);
                }
                else if (arg.Contains("--tcm"))
                {
                    tcm = splitOption(arg);
                }
                else if (arg.Contains("--tcmpool"))
                {
                    tcmpool = splitOption(arg);
                }
                else if (arg.Contains("--ttrmpool"))
                {
                    ttrmpool = splitOption(arg);
                }
                else if (arg.Contains("--ttrm"))
                {
                    ttrm = splitOption(arg);
                }
                else if (arg.Contains("--scm"))
                {
                    scm = splitOption(arg);
                }
                else if (arg.Contains("--scmpool"))
                {
                    scmpool = splitOption(arg);
                }
                else if (arg.Contains("--notes"))
                {
                    user.Notes = splitOption(arg);
                }
            }

            user.Licenses = parseLicenses(ttpro, ttpropool, tcm, tcmpool, ttrm, ttrmpool, scm, scmpool, user.Licenses);

            return user;
        }

        static private Products parseProduct(string prod)
        {
            Products product = Products.kTTP;
            if(prod.Contains("ttpro"))
            {
                product = Products.kTTP;
            }
            else if(prod.Contains("tcm"))
            {
                product = Products.kTCM;
            }
            else if (prod.Contains("ttrm"))
            {
                product = Products.kRM;
            }
            else if (prod.Contains("scm"))
            {
                product = Products.kSCM;
            }

            return product;
        }

        static private CPool parsePool(List<string> args, CPool inPool)
        {
            CPool pool = null;

            if (inPool == null)
            {
                pool = new CPool();
            }
            else
            {
                pool = new CPool(inPool);
            }

            foreach(string arg in args)
            {
                if (arg.Contains("--id"))
                {
                    pool.ID = System.Convert.ToUInt32(splitOption(arg));
                }
                else if(arg.Contains("--name"))
                {
                    pool.Name = splitOption(arg);
                }
                else if(arg.Contains("--product"))
                {
                    pool.Product = parseProduct(splitOption(arg));
                }
            }

            return pool;
        }

        static void Main(string[] args)
        {
            splscli app = new splscli(args);

            try
            {
                app.executeCommand();
            }
            catch (System.Exception ex)
            {
                Console.WriteLine(ex.Message);
                splscli.printHelp();
#if DEBUG
                Console.WriteLine(ex.StackTrace);
#endif
            }
        }
    }
}
